@@ -7,28 +7,28 @@
 import sys
 from collections import deque
 from threading import Lock
+from gi.repository import GObject
 
 from xpra.sound.sound_pipeline import SoundPipeline
-from xpra.gtk_common.gobject_util import one_arg_signal, gobject
+from xpra.gtk_common.gobject_util import one_arg_signal
 from xpra.sound.gstreamer_util import (
     plugin_str, get_decoder_elements,
     get_queue_time, normv, get_decoders,
     get_default_sink, get_sink_plugins,
     MP3, CODEC_ORDER, gst, QUEUE_LEAK,
     GST_QUEUE_NO_LEAK, MS_TO_NS, DEFAULT_SINK_PLUGIN_OPTIONS,
+    GST_FLOW_OK,
     )
-from xpra.gtk_common.gobject_compat import import_glib
 from xpra.net.compression import decompress_by_name
 from xpra.scripts.config import InitExit
 from xpra.util import csv, envint, envbool
-from xpra.os_util import thread, monotonic_time
+from xpra.os_util import monotonic_time
+from xpra.make_thread import start_thread
 from xpra.log import Logger
 
 log = Logger("sound")
 gstlog = Logger("gstreamer")
 
-glib = import_glib()
-
 
 SINK_SHARED_DEFAULT_ATTRIBUTES = {"sync"    : False,
                                   "async"   : True,
@@ -66,7 +66,7 @@ class SoundSink(SoundPipeline):
         "eos"       : one_arg_signal,
         })
 
-    def __init__(self, sink_type=None, sink_options={}, codecs=get_decoders(), codec_options={}, volume=1.0):
+    def __init__(self, sink_type=None, sink_options=None, codecs=(), codec_options=None, volume=1.0):
         if not sink_type:
             sink_type = get_default_sink()
         if sink_type not in get_sink_plugins():
@@ -74,14 +74,16 @@ class SoundSink(SoundPipeline):
         matching = [x for x in CODEC_ORDER if (x in codecs and x in get_decoders())]
         log("SoundSink(..) found matching codecs %s", matching)
         if not matching:
-            raise InitExit(1, "no matching codecs between arguments '%s' and supported list '%s'" % (csv(codecs), csv(get_decoders().keys())))
+            raise InitExit(1, "no matching codecs between arguments '%s' and supported list '%s'" % (
+                csv(codecs), csv(get_decoders().keys())))
         codec = matching[0]
         decoder, parser, stream_compressor = get_decoder_elements(codec)
-        SoundPipeline.__init__(self, codec)
+        super().__init__(codec)
         self.container_format = (parser or "").replace("demux", "").replace("depay", "")
         self.sink_type = sink_type
         self.stream_compressor = stream_compressor
-        log("container format=%s, stream_compressor=%s, sink type=%s", self.container_format, self.stream_compressor, self.sink_type)
+        log("container format=%s, stream_compressor=%s, sink type=%s",
+            self.container_format, self.stream_compressor, self.sink_type)
         self.levels = deque(maxlen=100)
         self.volume = None
         self.src    = None
@@ -135,7 +137,8 @@ class SoundSink(SoundPipeline):
             v = get_options_cb()
             log("%s()=%s", get_options_cb, v)
             sink_attributes.update(v)
-        sink_attributes.update(sink_options)
+        if sink_options:
+            sink_attributes.update(sink_options)
         sink_str = plugin_str(sink_type, sink_attributes)
         pipeline_els.append(sink_str)
         if not self.setup_pipeline_and_bus(pipeline_els):
@@ -168,13 +171,13 @@ class SoundSink(SoundPipeline):
 
     def start_adjust_volume(self, interval=100):
         if self.volume_timer!=0:
-            glib.source_remove(self.volume_timer)
+            self.source_remove(self.volume_timer)
         self.volume_timer = self.timeout_add(interval, self.adjust_volume)
         return False
 
     def cancel_volume_timer(self):
         if self.volume_timer!=0:
-            glib.source_remove(self.volume_timer)
+            self.source_remove(self.volume_timer)
             self.volume_timer = 0
 
 
@@ -210,7 +213,7 @@ class SoundSink(SoundPipeline):
         now = monotonic_time()
         if self.queue_state=="starting" or 1000*(now-self.start_time)<GRACE_PERIOD:
             gstlog("ignoring underrun during startup")
-            return 1
+            return True
         self.underruns += 1
         gstlog("queue_underrun")
         self.queue_state = "underrun"
@@ -225,7 +228,7 @@ class SoundSink(SoundPipeline):
                 self.set_max_level()
                 self.set_min_level()
         self.emit_info()
-        return 1
+        return True
 
     def get_level_range(self, mintime=2, maxtime=10):
         now = monotonic_time()
@@ -235,13 +238,13 @@ class SoundSink(SoundPipeline):
             minl = min(filtered)
             #range of the levels recorded:
             return maxl-minl
-        return 0
+        return GST_FLOW_OK
 
     def queue_overrun(self, *_args):
         now = monotonic_time()
         if self.queue_state=="starting" or 1000*(now-self.start_time)<GRACE_PERIOD:
             gstlog("ignoring overrun during startup")
-            return 1
+            return True
         clt = self.queue.get_property("current-level-time")//MS_TO_NS
         log("queue_overrun level=%ims", clt)
         now = monotonic_time()
@@ -253,7 +256,7 @@ class SoundSink(SoundPipeline):
             self.set_max_level()
             self.overrun_events.append(now)
         self.overruns += 1
-        return 1
+        return True
 
     def set_min_level(self):
         if not self.queue:
@@ -271,7 +274,8 @@ class SoundSink(SoundPipeline):
             mst = self.queue.get_property("max-size-time")//MS_TO_NS
             mrange = max(lrange+100, UNDERRUN_MIN_LEVEL)
             mtt = min(mst-50, mrange)
-            gstlog("set_min_level mtt=%3i, max-size-time=%3i, lrange=%s, mrange=%s (UNDERRUN_MIN_LEVEL=%s)", mtt, mst, lrange, mrange, UNDERRUN_MIN_LEVEL)
+            gstlog("set_min_level mtt=%3i, max-size-time=%3i, lrange=%s, mrange=%s (UNDERRUN_MIN_LEVEL=%s)",
+                   mtt, mst, lrange, mrange, UNDERRUN_MIN_LEVEL)
         else:
             mtt = 0
         cmtt = self.queue.get_property("min-threshold-time")//MS_TO_NS
@@ -335,9 +339,9 @@ class SoundSink(SoundPipeline):
         if self.src:
             self.src.emit('end-of-stream')
         self.cleanup()
-        return 0
+        return GST_FLOW_OK
 
-    def get_info(self):
+    def get_info(self) -> dict:
         info = SoundPipeline.get_info(self)
         if QUEUE_TIME>0 and self.queue:
             clt = self.queue.get_property("current-level-time")
@@ -391,7 +395,7 @@ class SoundSink(SoundPipeline):
                 clt = self.queue.get_property("current-level-time")//MS_TO_NS
                 qmin = self.queue.get_property("min-threshold-time")//MS_TO_NS
                 gstlog("add_data: refill=%s, level=%i, min=%i", self.refill, clt, qmin)
-                if qmin>0 and clt>qmin:
+                if 0<qmin<clt:
                     self.refill = False
         self.emit_info()
 
@@ -411,7 +415,7 @@ class SoundSink(SoundPipeline):
                 d = normv(d)
                 if d>0:
                     buf.duration = normv(d)
-        if self.push_buffer(buf):
+        if self.push_buffer(buf)==GST_FLOW_OK:
             self.inc_buffer_count()
             self.inc_byte_count(len(data))
             return True
@@ -434,18 +438,19 @@ class SoundSink(SoundPipeline):
         #buf.offset_end = offset_end
         #buf.set_caps(gst.caps_from_string(caps))
         r = self.src.emit("push-buffer", buf)
-        if r!=gst.FlowReturn.OK:
-            if self.queue_state != "error":
-                log.error("Error pushing buffer: %s", r)
-                self.update_state("error")
-                self.emit('error', "push-buffer error: %s" % r)
-            return 0
+        if r==gst.FlowReturn.OK:
+            return r
+        if self.queue_state!="error":
+            log.error("Error pushing buffer: %s", r)
+            self.update_state("error")
+            self.emit('error', "push-buffer error: %s" % r)
         return 1
 
-gobject.type_register(SoundSink)
+GObject.type_register(SoundSink)
 
 
 def main():
+    from gi.repository import GLib
     from xpra.platform import program_context
     with program_context("Sound-Record"):
         args = sys.argv
@@ -490,16 +495,16 @@ def main():
         ss = SoundSink(codecs=codecs)
         def eos(*args):
             print("eos%s" % (args,))
-            glib.idle_add(glib_mainloop.quit)
+            GLib.idle_add(glib_mainloop.quit)
         ss.connect("eos", eos)
         ss.start()
 
-        glib_mainloop = glib.MainLoop()
+        glib_mainloop = GLib.MainLoop()
 
         import signal
         def deadly_signal(*_args):
-            glib.idle_add(ss.stop)
-            glib.idle_add(glib_mainloop.quit)
+            GLib.idle_add(ss.stop)
+            GLib.idle_add(glib_mainloop.quit)
             def force_quit(_sig, _frame):
                 sys.exit()
             signal.signal(signal.SIGINT, force_quit)
@@ -511,12 +516,12 @@ def main():
             qtime = ss.queue.get_property("current-level-time")//MS_TO_NS
             if qtime<=0:
                 log.info("underrun (end of stream)")
-                thread.start_new_thread(ss.stop, ())
-                glib.timeout_add(500, glib_mainloop.quit)
+                start_thread(ss.stop, "stop", daemon=True)
+                GLib.timeout_add(500, glib_mainloop.quit)
                 return False
             return True
-        glib.timeout_add(1000, check_for_end)
-        glib.idle_add(ss.add_data, data)
+        GLib.timeout_add(1000, check_for_end)
+        GLib.idle_add(ss.add_data, data)
 
         glib_mainloop.run()
         return 0
